package com.gic.haoban.manage.service.dao.mapper.role;

import com.gic.haoban.manage.service.entity.role.TabHaobanRole;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 好办小程序权限(TabHaobanRole)表数据库访问层
 *
 * @author mozhu
 * @since 2022-09-06 16:35:04
 */
public interface TabHaobanRoleMapper {

    /**
     * 通过ID查询单条数据
     *
     * @param roleId 主键
     * @return 实例对象
     */
    TabHaobanRole queryById(Long roleId);

    /**
     * 新增数据
     *
     * @param tabHaobanRole 实例对象
     * @return 影响行数
     */
    int insert(TabHaobanRole tabHaobanRole);

    /**
     * 批量新增数据（MyBatis原生foreach方法）
     *
     * @param entities List<TabHaobanRole> 实例对象列表
     * @return 影响行数
     */
    int insertBatch(@Param("entities") List<TabHaobanRole> entities);

    /**
     * 修改数据
     *
     * @param tabHaobanRole 实例对象
     * @return 影响行数
     */
    int update(TabHaobanRole tabHaobanRole);

    /**
     * 按角色删除id
     *
     * @param roleId 角色id
     * @return int
     * @author mozhu
     * @date 2022-09-14 10:13:03
     */
    int deleteByRoleId(Long roleId);

    /**
     * 根据类型查询企业的菜单权限
     *
     * @param wxEnterpriseId wx企业标识
     * @param clerkType      职员类型
     * @return {@link List }<{@link TabHaobanRole }>
     * @author mozhu
     * @date 2022-09-07 14:09:07
     */
    TabHaobanRole getByWxEnterpriseIdAndClerkType(@Param("wxEnterpriseId") String wxEnterpriseId,
                                                        @Param("clerkType") Integer clerkType,
                                                  @Param("systemFlag") Integer systemFlag);


    /**
     * 根据wx企业id获取数量
     *
     * @param wxEnterpriseId wx企业标识
     * @return int
     * @author mozhu
     * @date 2022-09-14 10:36:05
     */
    int getCountByWxEnterpriseId(@Param("wxEnterpriseId") String wxEnterpriseId,@Param("systemFlag") Integer systemFlag);


    /**
     * 根据wx企业id获取列表
     *
     * @param wxEnterpriseId wx企业标识
     * @return {@link List}<{@link TabHaobanRole}>
     */
    List<TabHaobanRole> getListByWxEnterpriseId(@Param("wxEnterpriseId") String wxEnterpriseId,@Param("systemFlag") Integer systemFlag);

    /**
     * 根据wx企业id 删除
     *
     * @param wxEnterpriseId wx企业标识
     * @return int
     * @author mozhu
     * @date 2022-09-14 10:17:21
     */
    int deleteByWxEnterpriseId(String wxEnterpriseId,@Param("systemFlag") Integer systemFlag);
}

