package com.gic.haoban.manage.service.service.impl;

import com.gic.commons.util.UniqueIdUtils;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.manage.api.dto.qdto.QywxErrorLogListQDTO;
import com.gic.haoban.manage.service.dao.mapper.TabQywxErrorLogMapper;
import com.gic.haoban.manage.service.entity.TabHaobanWxApplication;
import com.gic.haoban.manage.service.entity.TabQywxErrorLog;
import com.gic.haoban.manage.service.pojo.bo.TabQywxErrorLogBO;
import com.gic.haoban.manage.service.service.QywxErrorLogService;
import com.gic.haoban.manage.service.service.WxApplicationService;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * 企微报错日志(TabQywxErrorLog)表服务实现类
 *
 * @author mozhu
 * @date 2022-04-18 15:28:02
 */
@Service
public class TabQywxErrorLogServiceImpl implements QywxErrorLogService {
    private static final Logger logger = LogManager.getLogger(TabQywxErrorLogServiceImpl.class);

    @Autowired
    private TabQywxErrorLogMapper tabQywxErrorLogMapper;
    @Autowired
    private WxApplicationService wxApplicationService;


    @Override
    public TabQywxErrorLogBO insert(TabQywxErrorLogBO tabQywxErrorLogBO) {
        String wxEnterpriseId = tabQywxErrorLogBO.getWxEnterpriseId();
        String corpid = tabQywxErrorLogBO.getCorpid();
        if (StringUtils.isBlank(wxEnterpriseId)) {
            TabHaobanWxApplication tabHaobanWxApplication = wxApplicationService.selectByCorpId(corpid);
            if (tabHaobanWxApplication != null) {
                tabQywxErrorLogBO.setWxEnterpriseId(tabHaobanWxApplication.getWxEnterpriseId());
            }
        }
        wxEnterpriseId = tabQywxErrorLogBO.getWxEnterpriseId();
        if (StringUtils.isBlank(wxEnterpriseId)) {
            logger.error("企业微信id为空,corpid:{}", corpid);
            return null;
        }
        tabQywxErrorLogBO.setId(UniqueIdUtils.uniqueLong());
        tabQywxErrorLogBO.setCreateTime(new Date());
        this.tabQywxErrorLogMapper.insert(EntityUtil.changeEntityByOrika(TabQywxErrorLog.class, tabQywxErrorLogBO));
        return tabQywxErrorLogBO;
    }

    @Override
    public TabQywxErrorLogBO queryById(Long id) {
        TabQywxErrorLog tabQywxErrorLog = this.tabQywxErrorLogMapper.queryById(id);
        return EntityUtil.changeEntityByOrika(TabQywxErrorLogBO.class, tabQywxErrorLog);
    }

    @Override
    public List<TabQywxErrorLog> queryListByPage(QywxErrorLogListQDTO qywxErrorLogListQDTO) {
        return this.tabQywxErrorLogMapper.queryListByPage(qywxErrorLogListQDTO);
    }

}
