package com.gic.haoban.manage.service.service.out.impl.fee;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.dubbo.entity.ProviderLocalTag;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.constants.Manage3Constants;
import com.gic.haoban.manage.api.dto.AlertMessageDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseQwDTO;
import com.gic.haoban.manage.api.dto.fee.HaobanQywxFeeAccountInterceptLogDTO;
import com.gic.haoban.manage.api.dto.fee.HaobanQywxFeeDTO;
import com.gic.haoban.manage.api.dto.fee.HaobanQywxFeeDetailDTO;
import com.gic.haoban.manage.api.dto.qdto.QywxFeeListQDTO;
import com.gic.haoban.manage.api.enums.AlertTypeEnum;
import com.gic.haoban.manage.api.service.fee.HaobanQywxFeeApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterprise;
import com.gic.haoban.manage.service.entity.fee.*;
import com.gic.haoban.manage.service.pojo.bo.fee.HaobanQywxFeeBO;
import com.gic.haoban.manage.service.service.StaffService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.service.fee.*;
import com.gic.mq.sdk.GicMQClient;
import com.gic.wechat.api.dto.qywx.fee.*;
import com.gic.wechat.api.dto.qywx.fee.qdto.FeeOrderListQDTO;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;

@Service("haobanQywxFeeApiService")
public class HaobanQywxFeeApiServiceImpl implements HaobanQywxFeeApiService {

    private static Logger logger = LoggerFactory.getLogger(HaobanQywxFeeApiServiceImpl.class);

    @Autowired
    private HaobanQywxFeeService haobanQywxFeeService;
    @Autowired
    private HaobanQywxFeeOrderService haobanQywxFeeOrderService;
    @Autowired
    private HaobanQywxFeeOrderAccountService haobanQywxFeeOrderAccountService;
    @Autowired
    private HaobanQywxFeeAccountStaffService haobanQywxFeeAccountStaffService;
    @Autowired
    private QywxUserApiService qywxUserApiService;
    @Autowired
    private Config config;
    @Autowired
    private WxEnterpriseService wxEnterpriseService;
    @Autowired
    private StaffService staffService;
    @Autowired
    private HaobanQywxFeeAccountInterceptLogService haobanQywxFeeAccountInterceptLogService;

    @Override
    public void run(String param) {
        List<TabHaobanWxEnterprise> tabHaobanWxEnterprises = wxEnterpriseService.listAll();
        for (TabHaobanWxEnterprise wxEnterprise : tabHaobanWxEnterprises) {
            GicMQClient instance = GICMQClientUtil.getClientInstance();
            try {
                instance.sendMessage("qywxAddFeeMq", wxEnterprise.getWxEnterpriseId());
            } catch (Exception e) {
                logger.info("发送消息处理异常：{}", e.getMessage(), e);
            }
        }
    }

    @Override
    public void addFee(String wxEnterpriseId) {
        if (StringUtils.isBlank(wxEnterpriseId)) {
            logger.error("wx企业id为空");
            return;
        }
        String serviceCorpid = config.getCorpid();
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        WxEnterpriseQwDTO qwDTO = this.wxEnterpriseService.getQwInfo(wxEnterpriseId);
        String corpid = qwDTO.getThirdCorpid();
        String corpName = wxEnterpriseDTO.getCorpName();
        String autoActiveStatus = qywxUserApiService.getAutoActiveStatus(corpid, serviceCorpid);
        if (!"1".equals(autoActiveStatus)) {
            logger.info("未开启自动激活：{}", corpid);
            wxEnterpriseService.updateAutoActiveFlagById(0, wxEnterpriseId);
            return;
        }
        logger.info("企微收费：{},{}", corpid, wxEnterpriseId);
        boolean hasOrder = haobanQywxFeeOrderService.hasOrder(wxEnterpriseId);
        //更新订单
        List<String> orderIds = new ArrayList<>();
        while (true) {
            FeeOrderListQDTO feeOrderListQDTO = new FeeOrderListQDTO();
            feeOrderListQDTO.setCorpid(corpid);
            if (hasOrder) {
                feeOrderListQDTO.setStartTime(DateUtil.beginOfDay(new Date()).offsetNew(DateField.HOUR_OF_DAY, -1).getTime());
                feeOrderListQDTO.setEndTime(DateUtil.endOfDay(new Date()).offsetNew(DateField.HOUR_OF_DAY, -1).getTime());
            }
            feeOrderListQDTO.setLimit(Manage3Constants.QW_LIMIT);
            FeeOrderResponseList feeOrderResponseList = qywxUserApiService.listOrder(corpid, serviceCorpid, feeOrderListQDTO);
            int errcode = feeOrderResponseList.getErrcode();
            if (errcode != 0) {
                logger.error("查询订单列表失败：{}", feeOrderResponseList.getErrmsg());
                break;
            }
            for (FeeOrderResponseList.OrderListDTO orderListDTO : feeOrderResponseList.getOrderList()) {
                orderIds.add(orderListDTO.getOrderId());
            }
            Integer hasMore = feeOrderResponseList.getHasMore();
            if (hasMore == 0) {
                logger.error("订单无更多账号");
                break;
            }
            feeOrderListQDTO.setCursor(feeOrderResponseList.getNextCursor());
            feeOrderResponseList = qywxUserApiService.listOrder(corpid, serviceCorpid, feeOrderListQDTO);
            for (FeeOrderResponseList.OrderListDTO orderListDTO : feeOrderResponseList.getOrderList()) {
                orderIds.add(orderListDTO.getOrderId());
            }
        }

        List<Map<String, String>> activeCodeList = new ArrayList<>();
        for (String orderId : orderIds) {
            FeeOrderDeatilResponseDTO order = qywxUserApiService.getOrder(corpid, serviceCorpid, orderId);
            if (order.getErrcode() != 0) {
                logger.error("请求企微失败：{}", orderId);
                break;
            }

            //插入订单
            FeeOrderDeatilResponseDTO.OrderDTO orderDTO = order.getOrder();
            Integer orderStatus = orderDTO.getOrderStatus();
            if (orderStatus != 1) {
                logger.info("只处理已支付订单:{}", orderId);
                continue;
            }

            TabHaobanQywxFeeOrder tabHaobanQywxFeeOrder = new TabHaobanQywxFeeOrder();
            tabHaobanQywxFeeOrder.setFeeOrderId(UniqueIdUtils.uniqueLong());
            tabHaobanQywxFeeOrder.setOrderId(orderId);
            tabHaobanQywxFeeOrder.setCorpId(corpid);
            tabHaobanQywxFeeOrder.setWxEnterpriseId(wxEnterpriseId);
            tabHaobanQywxFeeOrder.setOrderType(orderDTO.getOrderType());
            tabHaobanQywxFeeOrder.setOrderStatus(orderStatus);
            tabHaobanQywxFeeOrder.setPrice(new BigDecimal(orderDTO.getPrice()));
            tabHaobanQywxFeeOrder.setBaseCount(orderDTO.getAccountCount().getBaseCount());
            tabHaobanQywxFeeOrder.setExternalContactCount(orderDTO.getAccountCount().getExternalContactCount());
            tabHaobanQywxFeeOrder.setMonths(orderDTO.getAccountDuration().getMonths());
            tabHaobanQywxFeeOrder.setCreateTime(DateUtil.date(orderDTO.getCreateTime() * 1000));
            tabHaobanQywxFeeOrder.setPayTime(DateUtil.date(orderDTO.getPayTime() * 1000));
            haobanQywxFeeOrderService.insert(tabHaobanQywxFeeOrder);

            FeeOrderListQDTO feeOrderListQDTO = new FeeOrderListQDTO();
            feeOrderListQDTO.setOrderId(orderId);
            feeOrderListQDTO.setLimit(Manage3Constants.QW_LIMIT);
            AccountListResponseDTO accountListResponseDTO = qywxUserApiService.listOrderAccount(corpid, serviceCorpid, feeOrderListQDTO);

            if (accountListResponseDTO.getErrcode() != 0) {
                logger.info("订单账号列表报错：{}", accountListResponseDTO.getErrmsg());
                break;
            }

            for (AccountListDTO accountListDTO : accountListResponseDTO.getAccountList()) {
                Map<String, String> map = new HashMap<>();
                map.put("orderId", orderId);
                map.put("activeCode", accountListDTO.getActiveCode());
                activeCodeList.add(map);
            }

            Integer hasMore = accountListResponseDTO.getHasMore();
            if (hasMore == 0) {
                logger.error("订单无更多账号");
                break;
            }

            feeOrderListQDTO.setCursor(feeOrderListQDTO.getCursor());
            accountListResponseDTO = qywxUserApiService.listOrderAccount(corpid, serviceCorpid, feeOrderListQDTO);
            for (AccountListDTO accountListDTO : accountListResponseDTO.getAccountList()) {
                Map<String, String> map = new HashMap<>();
                map.put("orderId", orderId);
                map.put("activeCode", accountListDTO.getActiveCode());
                activeCodeList.add(map);
            }
        }

        if (CollectionUtils.isNotEmpty(activeCodeList)) {
            //处理订单账号
            addOrderAccount(wxEnterpriseId, serviceCorpid, corpid, activeCodeList);
        }

        List<String> userIdList = new ArrayList<>();
        //企业账号
        while (true) {
            FeeOrderListQDTO feeOrderListQDTO = new FeeOrderListQDTO();
            feeOrderListQDTO.setCorpid(corpid);
            feeOrderListQDTO.setLimit(Manage3Constants.QW_LIMIT);
            AccountListResponseDTO accountListResponseDTO = qywxUserApiService.listActivedAccount(corpid, serviceCorpid, feeOrderListQDTO);

            if (accountListResponseDTO.getErrcode() != 0) {
                logger.info("订单账号列表报错：{}", accountListResponseDTO.getErrmsg());
                break;
            }

            List<AccountListDTO> accountList = accountListResponseDTO.getAccountList();
            for (AccountListDTO accountListDTO : accountList) {
                userIdList.add(accountListDTO.getUserid());
            }

            Integer hasMore = accountListResponseDTO.getHasMore();
            if (hasMore == 0) {
                logger.error("无更多账号");
                break;
            }
            feeOrderListQDTO.setCursor(feeOrderListQDTO.getCursor());
            accountListResponseDTO = qywxUserApiService.listActivedAccount(corpid, serviceCorpid, feeOrderListQDTO);
            for (AccountListDTO accountListDTO : accountListResponseDTO.getAccountList()) {
                userIdList.add(accountListDTO.getUserid());
            }
        }
        if (CollectionUtils.isNotEmpty(userIdList)) {
            addAccountStaff(wxEnterpriseId, serviceCorpid, corpid, userIdList);
        }
        Integer autoActiveFlag = wxEnterpriseDTO.getAutoActiveFlag();
        if (autoActiveFlag != null && autoActiveFlag != 1) {
            //更新企微状态
            wxEnterpriseService.updateAutoActiveFlagById(1, wxEnterpriseId);
        }

        //更新收费
        HaobanQywxFeeDetailDTO haobanQywxFeeDetailDTO = haobanQywxFeeAccountStaffService.feeIntercept(wxEnterpriseId);
        int permissionUserNum = haobanQywxFeeOrderAccountService.countOrderAccountStatus(1, wxEnterpriseId);
        if (haobanQywxFeeDetailDTO != null && permissionUserNum > 0) {
            TabHaobanQywxFee tabHaobanQywxFee = new TabHaobanQywxFee();
            tabHaobanQywxFee.setQwFeeId(UniqueIdUtils.uniqueLong());
            tabHaobanQywxFee.setCorpName(corpName);
            tabHaobanQywxFee.setCorpId(corpid);
            tabHaobanQywxFee.setWxEnterpriseId(wxEnterpriseId);
            tabHaobanQywxFee.setInterceptTime(haobanQywxFeeDetailDTO.getInterceptTime());
            tabHaobanQywxFee.setInterceptUserNum(haobanQywxFeeDetailDTO.getInterceptUserNum());
            tabHaobanQywxFee.setPermissionUserNum(permissionUserNum);
            tabHaobanQywxFee.setBuyPermissionUserNum(haobanQywxFeeOrderService.countExternalContactCount(wxEnterpriseId));
            tabHaobanQywxFee.setInterceptFlag(0);
            haobanQywxFeeService.insert(tabHaobanQywxFee);
            //判断今天是否是本月最后一天
            if (DateUtil.isSameDay(DateUtil.endOfMonth(new Date()), new Date())) {
                int monthWillInterceptNum = haobanQywxFeeAccountStaffService.monthWillInterceptNum(wxEnterpriseId);
                String content = "30天将有" + monthWillInterceptNum + "个接口许可证即将过期，您总共还剩余" + permissionUserNum + "个可用的接口许可证，请保证数量充足，如需增加请及时联系相关人员续费，避免影响使用";
                //告警
                messageAlert(wxEnterpriseId, corpName, corpid, AlertTypeEnum.FEE_ALARM.getName(), content);
            }
        }

        List<TabHaobanQywxFeeAccountInterceptLog> tabHaobanQywxFeeAccountInterceptLogs = haobanQywxFeeAccountInterceptLogService.queryByWxEnterpriseId(wxEnterpriseId);
        if (CollectionUtils.isNotEmpty(tabHaobanQywxFeeAccountInterceptLogs)) {
            TabHaobanQywxFee tabHaobanQywxFee = new TabHaobanQywxFee();
            tabHaobanQywxFee.setQwFeeId(UniqueIdUtils.uniqueLong());
            tabHaobanQywxFee.setCorpName(corpName);
            tabHaobanQywxFee.setCorpId(corpid);
            tabHaobanQywxFee.setWxEnterpriseId(wxEnterpriseId);
            tabHaobanQywxFee.setInterceptTime(DateUtil.offsetDay(new Date(), -1));
            tabHaobanQywxFee.setInterceptUserNum(tabHaobanQywxFeeAccountInterceptLogs.size());
            tabHaobanQywxFee.setPermissionUserNum(permissionUserNum);
            tabHaobanQywxFee.setBuyPermissionUserNum(haobanQywxFeeOrderService.countExternalContactCount(wxEnterpriseId));
            tabHaobanQywxFee.setInterceptFlag(1);
            haobanQywxFeeService.insert(tabHaobanQywxFee);
        }
    }

    private void addOrderAccount(String wxEnterpriseId, String serviceCorpid, String corpid, List<Map<String, String>> activeCodeList) {
        if (CollectionUtils.isEmpty(activeCodeList)) {
            logger.error("订单账号无激活码");
            return;
        }
        for (Map<String, String> activeCodeMap : activeCodeList) {
            String activeCode = activeCodeMap.get("activeCode");
            String orderId = activeCodeMap.get("orderId");
            ActiveInfoCodeResponseDTO activeInfoCodeResponseDTO = qywxUserApiService.getActiveInfoByCode(corpid, serviceCorpid, activeCode);
            if (activeInfoCodeResponseDTO.getErrcode() != 0) {
                logger.error("激活码详情报错：{}", activeInfoCodeResponseDTO.getErrmsg());
                continue;
            }
            String userid = activeInfoCodeResponseDTO.getUserid();
            if (StringUtils.isNotBlank(userid)) {
                TabHaobanQywxFeeOrderAccount tabHaobanQywxFeeOrderAccount = new TabHaobanQywxFeeOrderAccount();
                tabHaobanQywxFeeOrderAccount.setFeeAccountId(UniqueIdUtils.uniqueLong());
                tabHaobanQywxFeeOrderAccount.setOrderId(orderId);
                tabHaobanQywxFeeOrderAccount.setCorpId(corpid);
                tabHaobanQywxFeeOrderAccount.setWxEnterpriseId(wxEnterpriseId);
                tabHaobanQywxFeeOrderAccount.setWxUserId(userid);
                tabHaobanQywxFeeOrderAccount.setActiveCode(activeInfoCodeResponseDTO.getActiveCode());
                tabHaobanQywxFeeOrderAccount.setAccountType(activeInfoCodeResponseDTO.getType());
                tabHaobanQywxFeeOrderAccount.setStatus(activeInfoCodeResponseDTO.getStatus());
                tabHaobanQywxFeeOrderAccount.setExpireTime(DateUtil.date(activeInfoCodeResponseDTO.getExpireTime() * 1000));
                tabHaobanQywxFeeOrderAccount.setActiveTime(DateUtil.date(activeInfoCodeResponseDTO.getActiveTime() * 1000));
                haobanQywxFeeOrderAccountService.insert(tabHaobanQywxFeeOrderAccount);
            }
        }
    }

    private void addAccountStaff(String wxEnterpriseId, String serviceCorpid, String corpid, List<String> useridList) {
        if (CollectionUtils.isEmpty(useridList)) {
            logger.error("成员激活为空");
            return;
        }
        for (String userid : useridList) {
            //更新成员
            ActiveInfoUserResponseDTO activeInfoUserResponseDTO = qywxUserApiService.getActiveInfoByUser(corpid, serviceCorpid, userid);
            if (activeInfoUserResponseDTO.getErrcode() != 0) {
                logger.info("成员激活详情报错：{}", activeInfoUserResponseDTO.getErrmsg());
                return;
            }
            List<AccountListDTO> activeInfoList = activeInfoUserResponseDTO.getActiveInfoList();
            Integer activeStatus = activeInfoUserResponseDTO.getActiveStatus();
            TabHaobanStaff tabHaobanStaff = staffService.selectByUserIdAndEnterpriseId(userid, wxEnterpriseId);
            for (AccountListDTO accountListDTO : activeInfoList) {
                TabHaobanQywxFeeAccountStaff tabHaobanQywxFeeAccountStaff = new TabHaobanQywxFeeAccountStaff();
                tabHaobanQywxFeeAccountStaff.setFeeAccountStaffId(UniqueIdUtils.uniqueLong());
                tabHaobanQywxFeeAccountStaff.setCorpId(corpid);
                tabHaobanQywxFeeAccountStaff.setWxEnterpriseId(wxEnterpriseId);
                if (tabHaobanStaff != null) {
                    String staffId = tabHaobanStaff.getStaffId();
                    tabHaobanQywxFeeAccountStaff.setStaffId(staffId);
                }
                tabHaobanQywxFeeAccountStaff.setActiveCode(accountListDTO.getActiveCode());
                tabHaobanQywxFeeAccountStaff.setAccountType(accountListDTO.getType());
                tabHaobanQywxFeeAccountStaff.setExpireTime(DateUtil.date(accountListDTO.getExpireTime() * 1000));
                tabHaobanQywxFeeAccountStaff.setActiveTime(DateUtil.date(accountListDTO.getActiveTime() * 1000));
                haobanQywxFeeAccountStaffService.insert(tabHaobanQywxFeeAccountStaff);
            }
            if (tabHaobanStaff != null) {
                String staffId = tabHaobanStaff.getStaffId();
                staffService.updateActiveStatusById(activeStatus, staffId);
            }
        }
    }

    @Override
    public Page<HaobanQywxFeeDTO> listPage(QywxFeeListQDTO qywxFeeListQDTO) {
        Page<HaobanQywxFeeBO> haobanQywxFeeBOS = haobanQywxFeeService.listPage(qywxFeeListQDTO);
        return PageUtil.changeToCurrentPage(haobanQywxFeeBOS, HaobanQywxFeeDTO.class);
    }

    @Override
    public List<HaobanQywxFeeDetailDTO> feeDetail(String wxEnterpriseId) {
        return haobanQywxFeeAccountStaffService.feeDetail(wxEnterpriseId);
    }

    @Override
    public void addInterceptLog(String param) {
        HaobanQywxFeeAccountInterceptLogDTO haobanQywxFeeAccountInterceptLogDTO = JSON.toJavaObject(JSON.parseObject(param), HaobanQywxFeeAccountInterceptLogDTO.class);
        haobanQywxFeeAccountInterceptLogDTO.setFeeAccountInterceptId(UniqueIdUtils.uniqueLong());
        TabHaobanWxEnterprise haobanWxEnterprise = wxEnterpriseService.getEnterpriseBycorpId(haobanQywxFeeAccountInterceptLogDTO.getCorpId());
        if (haobanWxEnterprise != null) {
            String wxEnterpriseId = haobanWxEnterprise.getWxEnterpriseId();
            haobanQywxFeeAccountInterceptLogDTO.setWxEnterpriseId(wxEnterpriseId);
            TabHaobanStaff tabHaobanStaff = staffService.selectByUserIdAndEnterpriseId(haobanQywxFeeAccountInterceptLogDTO.getWxUserId(), wxEnterpriseId);
            if (tabHaobanStaff != null) {
                haobanQywxFeeAccountInterceptLogDTO.setStaffId(tabHaobanStaff.getStaffId());
            }
        }
        haobanQywxFeeAccountInterceptLogService.insert(EntityUtil.changeEntityNew(TabHaobanQywxFeeAccountInterceptLog.class, haobanQywxFeeAccountInterceptLogDTO));
    }

    @Override
    public HaobanQywxFeeDTO queryByWxEnterpriseId(String wxEnterpriseId) {
        return EntityUtil.changeEntityNew(HaobanQywxFeeDTO.class, haobanQywxFeeService.queryByWxEnterpriseId(wxEnterpriseId));
    }


    /**
     * 告警
     *
     * @param wxEnterpriseId
     */
    private void messageAlert(String wxEnterpriseId, String corpName, String corpid, String titleSub, String content) {
        ProviderLocalTag providerLocalTag = ProviderLocalTag.tag.get();
        String traceId = providerLocalTag.traceId;
        AlertMessageDTO messageDTO = new AlertMessageDTO();
        messageDTO.setAlertTitle(titleSub);
        messageDTO.setAlertType(AlertTypeEnum.FEE_ALARM.getType());
        messageDTO.setWxEnterpriseId(wxEnterpriseId);
        Map<String, Object> map = new HashMap<>();
        map.put("企业名称", corpName);
        map.put("企业id", corpid);
        map.put("内容", content);
        messageDTO.setContentMap(map);
        messageDTO.setTraceId(traceId);
        GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
        try {
            clientInstance.sendMessage("haobanAlertMq", JSONObject.toJSONString(messageDTO), 6 * 3600);
        } catch (Exception e) {
            logger.info("异常：{}", e.getMessage(), e);
        }
    }

}
