package com.gic.haoban.manage.service.task.friend;

import cn.hutool.crypto.SecureUtil;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.esotericsoftware.minlog.Log;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.haoban.manage.api.dto.DealParamMqDTO;
import com.gic.haoban.manage.api.dto.ExternalUserDTO;
import com.gic.haoban.manage.api.enums.PreDealStatusEnum;
import com.gic.haoban.manage.api.enums.PreDealTypeEnum;
import com.gic.haoban.manage.api.enums.SyncTaskStatusEnum;
import com.gic.haoban.manage.api.service.MemberUnionidRelatedApiService;
import com.gic.haoban.manage.service.entity.TabHaobanPreDealLog;
import com.gic.haoban.manage.service.exception.WxApiLimitException;
import com.gic.haoban.manage.service.service.PreDealService;
import com.gic.haoban.manage.service.service.SyncTaskService;
import com.gic.haoban.manage.service.task.BaseSyncOperation;
import com.gic.mq.sdk.GicMQClient;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 自建好友处理
 */
@Service
public class SelfFriendSyncNewOperation implements BaseSyncOperation {

    private static final Logger logger = LoggerFactory.getLogger(SelfFriendSyncNewOperation.class);
    @Autowired
    private PreDealService preDealService;
    @Autowired
    private MemberUnionidRelatedApiService memberUnionidRelatedApiService;
    @Autowired
    private SyncTaskService syncTaskService;
    @Autowired
    private FriendClerkSyncNewOperation friendClerkSyncNewOperation;

    private static final String TRY_AGAIN = "haoban_fresh_wx_friend_again_";

    @Override
    public void dealSingleByMq(DealParamMqDTO dealParamMqDTO, TabHaobanPreDealLog dataPre) {
        logger.info("同步好友开始,自建代开发={}", JSONObject.toJSONString(dataPre));
        String reason = "成功";
        boolean dealFlag = true;
        try {
            String wxEnterpriseId = dataPre.getWxEnterpriseId();
            String wxUserId = dataPre.getDataId().split("###")[1] ;
            String staffId = dataPre.getpDataId() ;
            String taskId = dealParamMqDTO.getTaskId();
            String externalUserId = dataPre.getDataContent();

            ExternalUserDTO externalUserDTO = memberUnionidRelatedApiService.getCorpSelfExternalUseridInfo(wxEnterpriseId, wxUserId, externalUserId);
            if (externalUserDTO == null) {
                logger.info("外部联系人好友不存在：{}", JSONObject.toJSONString(dataPre));
                dealException(taskId, dataPre.getDataId(), null, "好友不存在");
                return;
            }
            //自有应用信息
            JSONObject json = new JSONObject();
            json.put("name", externalUserDTO.getName());
            json.put("unionId", externalUserDTO.getUnionId());
            json.put("avatar", externalUserDTO.getAvatar());
            json.put("externalUserId", externalUserDTO.getExternalUserId());
            json.put("addCreateTime", externalUserDTO.getAddCreateTime());

            //relationkey
            logger.info("生成relationKey：{},{},{}",staffId,externalUserDTO.getName(),externalUserDTO.getAddCreateTime());
            String relationKey = SecureUtil.md5(staffId + "_" + externalUserDTO.getName() + "_" + externalUserDTO.getAddCreateTime());
            logger.info("relationKey：{}，{}", relationKey, externalUserDTO.getExternalUserId());
            //更新状态
            preDealService.updateFriendStatusByDataId(taskId, dataPre.getDataId(), PreDealStatusEnum.computed.getVal(), reason, json.toJSONString(), relationKey , PreDealTypeEnum.self_friend.getVal());
            dealSuccess(taskId, dataPre.getDataId(), null, wxEnterpriseId);
        } catch (WxApiLimitException e) {
            logger.info("接口次数限制：{}", JSONObject.toJSONString(dataPre));
            dealFlag = tryAgainToMq(dataPre);
            reason = "getCorpSelfExternalUseridInfo重试次数过多";
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("同步失败：{},{}", JSONObject.toJSONString(dataPre), e);
            reason = "自建应用好友处理异常";
            dealFlag = false;

        } finally {
            if (!dealFlag) {
                dealException(dealParamMqDTO.getTaskId(), dataPre.getDataId(), dataPre.getpDataId(), reason);
            }
        }
    }

    @Override
    public void dealException(String taskId, String dataId, String pDataId, String reason) {
        preDealService.updateStatusByDataId(taskId, dataId, PreDealStatusEnum.exception.getVal(), reason);
        checkDepartmentTask(taskId, pDataId);
    }

    @Override
    public void dealSuccess(String taskId, String dataId, String dataContent, String wxEnterpriseId) {
        TabHaobanPreDealLog data = preDealService.getByDataId(taskId, dataId, PreDealTypeEnum.self_friend.getVal(), -1);
        //校验成功是否都
        checkDepartmentTask(taskId, data.getpDataId());
    }

    /**
     * 重试机制
     *
     * @param dataPre
     * @return
     */
    private boolean tryAgainToMq(TabHaobanPreDealLog dataPre) {
        logger.info("需要重试self：{}", dataPre.getDataId());
        String key = TRY_AGAIN + dataPre.getTaskId() + ":" + dataPre.getDataId();
        Object cache = RedisUtil.getCache(key);
        if (null == cache) {
            RedisUtil.setCache(key, 1, 2L, TimeUnit.HOURS);
        } else {
            Integer count = Integer.valueOf(cache.toString());
            RedisUtil.setCache(key, count + 1);
            if (count > 4) {
                return false;
            }
        }
        HashSet<String> reTrysIds = new HashSet<>();
        reTrysIds.add(dataPre.getDataId());
        dealDepartmentToMq(dataPre.getTaskId(), reTrysIds, SyncTaskStatusEnum.self_friend_sync);
        return true;
    }

    /**
     * 自定义处理流程
     *
     * @param taskId
     * @param pDataId
     */
    public void checkDepartmentTask(String taskId, String pDataId) {
        //校验是否都已经处理完成
        boolean b = preDealService.checkFriendTaskByPDataId(taskId, pDataId, PreDealTypeEnum.self_friend.getVal());
        if (!b) {
            logger.info("校验直接返回");
            return;
        }
        logger.info("同步第三方服务商好友开始：{},{}", taskId, pDataId);
        String key = "haoban_sync_self_friend_task_" + taskId + ":" + pDataId;
        String lockKey = "haoban_sync_self_friend_task_lock_" + taskId + ":" + pDataId;
        RedisUtil.lock(lockKey, 3L);
        Object hasDealCheck = RedisUtil.getCache(key);
        if (hasDealCheck != null) {
            logger.info("重复提交处理第三方好友：{}", pDataId);
            RedisUtil.unlock(lockKey);
            return;
        }
        RedisUtil.setCache(key, pDataId, 30L, TimeUnit.SECONDS);
        List<String> dataIds = preDealService.listReDataIdByPDataId(taskId, pDataId, PreDealTypeEnum.friend.getVal());
        if (CollectionUtils.isEmpty(dataIds)) {
            logger.info("第三方的数据为空");
            //更新父级别
            preDealService.updateStatusByDataId(taskId, pDataId, PreDealTypeEnum.friend_clerk.getVal(), PreDealStatusEnum.computed.getVal(), "成功");
            //校验异常
            this.checkDepartmentTask(taskId);
        } else {
            dealDepartmentToMq(taskId, new HashSet<>(dataIds), SyncTaskStatusEnum.friend_sync);
        }
        RedisUtil.unlock(lockKey);
    }

    @Override
    public void checkDepartmentTask(String taskId) {
        friendClerkSyncNewOperation.checkDepartmentTask(taskId);
    }

    /**
     * 放入mq处理部门数据
     *
     * @param taskId
     * @param dealList
     */
    private void dealDepartmentToMq(String taskId, Set<String> dealList, SyncTaskStatusEnum syncTaskStatusEnum) {
        //预处理分组任务
        syncTaskService.updateTaskStatus(taskId, syncTaskStatusEnum.getVal());
        List<String> ret = dealList.stream().map(relationId -> {
            DealParamMqDTO dealParamMqDTO = new DealParamMqDTO();
            dealParamMqDTO.setData(relationId);
            dealParamMqDTO.setTaskId(taskId);
            dealParamMqDTO.setType(syncTaskStatusEnum.getVal());
            return JSONObject.toJSONString(dealParamMqDTO);
        }).collect(Collectors.toList());

        GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
        try {
            Log.info("发送队列SelfFriendSyncNewOperation={}",JSON.toJSONString(ret));
            clientInstance.sendBatchMessages("departmentSyncDealMq", ret);
        } catch (Exception e) {
            logger.info("发送失败:{},{}", taskId);
            e.printStackTrace();
        }
    }

}
